import {
  Body,
  Controller,
  Del,
  Get,
  Inject,
  Param,
  Post,
  Query,
} from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { ApplicationService } from '../service/application.service';
import {
  CDTO_application,
  CVO_application,
  FVO_application,
  // QDTO_application,
  QVO_application,
  UDTO_application,
  UVAO_application,
} from '../interface/application.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['比赛报名'])
@Controller('/application')
export class ApplicationController extends BaseController {
  @Inject()
  service: ApplicationService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CVO_application })
  async create(@Body() data: CDTO_application) {
    // 检查申请
    await this.service.checkApply(data);
    const dbData = await this.service.create(data);
    const result = new CVO_application(dbData);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_application })
  async query(@Query() filter: any) {
    const list = await this.service.specialQuery(filter);
    const data = [];
    for (const i of list.list) {
      const newData = new QVO_application(i);
      data.push(newData);
    }
    const total = list.total;
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FVO_application })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FVO_application(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UVAO_application })
  async update(@Param('id') id: string, @Body() body: UDTO_application) {
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
