import {
  Body,
  Controller,
  Del,
  Get,
  Inject,
  Param,
  Post,
  Query,
} from '@midwayjs/decorator';
import { BaseController } from 'free-midway-component';
import { TeamApplyService } from '../service/teamApply.service';
import {
  CDTO_teamApply,
  CVO_teamApply,
  FVO_teamApply,
  QDTO_teamApply,
  QVO_teamApply,
  UDTO_teamApply,
  UVAO_teamApply,
} from '../interface/teamApply.interface';
import { ApiResponse, ApiTags, ApiQuery } from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
@ApiTags(['团队申请'])
@Controller('/teamApply')
export class TeamApplyController extends BaseController {
  @Inject()
  service: TeamApplyService;

  @Post('/')
  @Validate()
  @ApiResponse({ type: CVO_teamApply })
  async create(@Body() data: CDTO_teamApply) {
    // 检查账号
    await this.service.checkApply(data);
    const dbData = await this.service.create(data);
    const result = new CVO_teamApply(dbData);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_teamApply })
  async query(
    @Query() filter: QDTO_teamApply,
    @Query('skip') skip: number,
    @Query('limit') limit: number
  ) {
    const list = await this.service.query(filter, { skip, limit });
    const data = [];
    for (const i of list) {
      const newData = new QVO_teamApply(i);
      data.push(newData);
    }
    const total = await this.service.count(filter);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FVO_teamApply })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FVO_teamApply(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UVAO_teamApply })
  async update(@Param('id') id: string, @Body() body: UDTO_teamApply) {
    const result = await this.service.update(id, body);
    return result;
  }

  @Get('/out')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: FVO_teamApply })
  async out(@Query() query: any) {
    await this.service.out(query);
    return 'ok';
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
