import { Provide } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import {
  BaseService,
  FrameworkErrorEnum,
  ServiceError,
} from 'free-midway-component';
import isEqual = require('lodash/isEqual');
import { User } from '../entity/user.entity';
import { LoginDTO } from '../interface/user.interface';
import { Team } from '../entity/team.entity';
type modelType = ReturnModelType<typeof User>;
@Provide()
export class UserService extends BaseService<modelType> {
  @InjectEntityModel(User)
  model: modelType;

  @InjectEntityModel(Team)
  TeamModel: ReturnModelType<typeof Team>;

  async findUserToLogin(data: LoginDTO): Promise<object> {
    const { account, password } = data;
    const user = await this.model.findOne({ account }, '+password').lean();
    if (!user) {
      throw new ServiceError(
        '未找到用户信息',
        FrameworkErrorEnum.NOT_FOUND_DATA
      );
    }
    if (user.status === '0') {
      throw new ServiceError(
        '用户审核中 请等待申请通过',
        FrameworkErrorEnum.NOT_FOUND_DATA
      );
    } else if (user.status === '-1') {
      throw new ServiceError(
        '用户审核拒绝 请重新申请！',
        FrameworkErrorEnum.NOT_FOUND_DATA
      );
    }
    if (!isEqual(user.password.secret, password))
      throw new ServiceError('密码错误', FrameworkErrorEnum.SERVICE_FAULT);
    return user;
  }
  // 团队人员
  async person(filter) {
    const { team } = filter;
    const list = await this.model.find({ status: '1', type: '0' });
    const result = [];
    for (const val of list) {
      const total = await this.TeamModel.count({ member: { $in: [val.id] } });
      if (total === 0) result.push(val);
    }
    if (team) {
      const teamInfo: any = await this.TeamModel.findById(team);
      if (teamInfo.member) {
        for (const val of teamInfo.member) {
          const list = await this.model.findById(val);
          if (list) result.push(list);
        }
      }
    }
    return result;
  }
}
