import { Provide } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { FrameworkErrorEnum, ServiceError } from 'free-midway-component';
import { User } from '../../entity/user.entity';
import {
  CheckUpdateCardAndPid,
  checkPhoneAndPid,
} from '../../interface/util/user.util.interface';

@Provide()
export class UserUtilService {
  @InjectEntityModel(User)
  userModel: ReturnModelType<typeof User>;

  /**
   * 检查创建时,账号
   * @param data 参数
   */
  async checkPhoneAndPid(data: checkPhoneAndPid) {
    if (data.account) {
      // 检查账号
      // 管理员
      const useran = await this.userModel.count({ account: data.account });
      if (useran > 0) {
        throw new ServiceError(
          '已有用户 使用该账号',
          FrameworkErrorEnum.BAD_BODY
        );
      }
    }
  }
  /**
   * 检查创建时,账号
   * @param id 当前要修改的数据
   * @param data 参数
   */
  async checkUpdateCardAndPid(id, data: CheckUpdateCardAndPid) {
    const { account } = data;
    if (account) {
      // 用户
      const personalan = await this.userModel.count({
        _id: { $ne: id },
        account,
      });
      if (personalan > 0)
        throw new ServiceError(
          `账号: ${account} ;已有个人用户使用该账号`,
          FrameworkErrorEnum.BAD_BODY
        );
    }
  }
}
